@@ -1,5 +1,5 @@
 package Set::ArrayAlt;
-# Copyright (c) 2009-2009 Sullivan Beck. All rights reserved.
+# Copyright (c) 2009-2010 Sullivan Beck. All rights reserved.
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
@@ -12,7 +12,7 @@ use strict;
 use Sort::DataTypes qw(sort_by_method sort_valid_method);
 
 use vars qw($VERSION);
-$VERSION = "1.00";
+$VERSION = "1.01";
 
 ###############################################################################
 # BASE METHODS
@@ -22,7 +22,7 @@ sub new {
    my($class,@array) = @_;
 
    my $self = {
-               "set"  => undef,
+               "set"  => [],
                "err"  => "",
               };
    bless $self, $class;
@@ -54,41 +54,77 @@ sub errmsg {
 # LIST EXAMINATION METHODS
 ###############################################################################
 
-sub list {
-   my($self,@list) = @_;
-   $$self{"err"}   = "";
-
-   if (@list) {
-      $$self{"set"} = [@list];
-      return;
-   } elsif (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   } else {
-      return @{ $$self{"set"} };
-   }
-}
-
-sub length {
-   my($self) = @_;
+sub as_hash {
+   my($self,$full) = @_;
    $$self{"err"} = "";
 
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
+   if ($full) {
 
-   return $#{ $$self{"set"} } + 1;
+      my %count;
+      my %vals;
+      my %refs;
+      my %scal;
+      my $undef;
+      my $label = 1;
+      foreach my $ele (@{ $$self{"set"} }) {
+         if (! defined $ele) {
+            if ($undef) {
+               $count{$undef}++;
+            } else {
+               $undef         = $label++;
+               $vals{$undef}  = undef;
+               $count{$undef} = 1;
+            }
+         } elsif (ref($ele)) {
+            my $s = scalar($ele);
+            my $l;
+            if (exists $refs{$s}) {
+               $l         = $refs{$s};
+               $count{$l}++;
+            } else {
+               $l         = $label++;
+               $refs{$s}  = $l;
+               $vals{$l}  = $ele;
+               $count{$l} = 1;
+            }
+         } else {
+            my $l;
+            if (exists $scal{$ele}) {
+               $l         = $scal{$ele};
+               $count{$l}++;
+            } else {
+               $l          = $label++;
+               $scal{$ele} = $l;
+               $vals{$l}   = $ele;
+               $count{$l}  = 1;
+            }
+         }
+      }
+
+      return (\%count, \%vals);
+
+   } else {
+
+      my %tmp;
+      foreach my $ele (@{ $$self{"set"} }) {
+         next  if (! defined $ele  ||  ref($ele));
+         if (exists $tmp{$ele}) {
+            $tmp{$ele}++;
+         } else {
+            $tmp{$ele} = 1;
+         }
+      }
+
+      return %tmp;
+
+   }
 }
 
 sub at {
    my($self,@n) = @_;
    $$self{"err"} = "";
 
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   } elsif (! @n) {
+   if      (! @n) {
       $$self{"err"} = "Index required";
       return undef;
    } elsif ($#n > 0  &&  ! wantarray) {
@@ -96,8 +132,14 @@ sub at {
       return undef;
    }
 
+   my @list = @{ $$self{"set"} };
+   if (! @list) {
+      $$self{"err"} = "Operation (at) invalid with empty list";
+      return undef;
+   }
+
    my(@ret);
-   my $len = $self->length();
+   my $len = $#list + 1;
 
    foreach my $n (@n) {
       if ($n =~ /^[+-]?\d+$/) {
@@ -105,7 +147,7 @@ sub at {
             $$self{"err"} = "Index out of range";
             return undef;
          }
-         CORE::push(@ret,$$self{"set"}[$n]);
+         CORE::push(@ret,$list[$n]);
       } else {
          $$self{"err"} = "Index must be an integer";
          return undef;
@@ -119,68 +161,6 @@ sub at {
    }
 }
 
-sub first {
-   my($self) = @_;
-   $$self{"err"} = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   return $$self{"set"}[0];
-}
-
-sub last {
-   my($self) = @_;
-   $$self{"err"} = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   return $$self{"set"}[$#{ $$self{"set"} }];
-}
-
-sub index {
-   my($self,$val) = @_;
-   $$self{"err"} = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   my @idx  = ();
-   my @list = @{ $$self{"set"} };
-
-   for (my $i=0; $i<=$#list; $i++) {
-      my $ele = $list[$i];
-      CORE::push(@idx,$i)
-          if ((defined $ele  &&  defined $val  &&  $ele eq $val)  ||
-              (! defined $val  &&  ! defined $ele));
-   }
-
-   if (wantarray) {
-      return @idx;
-   } else {
-      return $idx[0];
-   }
-}
-
-sub rindex {
-   my($self,$val) = @_;
-   my @idx = $self->index($val);
-   return undef  if ($self->err());
-
-   if (wantarray) {
-      return CORE::reverse(@idx);
-   } else {
-      return $idx[$#idx];
-   }
-}
-
 sub count {
    my($self,$val) = @_;
    my @idx = $self->index($val);
@@ -190,23 +170,81 @@ sub count {
 }
 
 sub exists {
+   my($self,@val) = @_;
+   @val = (undef)  if (! @val);
+
+   foreach my $val (@val) {
+      my @idx = $self->index($val);
+      return undef  if ($self->err());
+      return 0  if (! @idx);
+   }
+   return 1;
+}
+
+sub first {
+   my($self) = @_;
+   $$self{"err"} = "";
+
+   my @list = @{ $$self{"set"} };
+   if (! @list) {
+      $$self{"err"} = "Operation (first) invalid with empty list";
+      return undef;
+   }
+
+   return $list[0];
+}
+
+sub last {
+   my($self) = @_;
+   $$self{"err"} = "";
+
+   my @list = @{ $$self{"set"} };
+   if (! @list) {
+      $$self{"err"} = "Operation (first) invalid with empty list";
+      return undef;
+   }
+
+   return $list[$#list];
+}
+
+sub index {
+   my($self,$val) = @_;
+   $$self{"err"} = "";
+
+   my @idx  = ();
+   my @list = @{ $$self{"set"} };
+
+   for (my $i=0; $i<=$#list; $i++) {
+      my $ele = $list[$i];
+      CORE::push(@idx,$i)  if (_eq($self,$val,$ele));
+   }
+
+   if (wantarray) {
+      return @idx;
+   } elsif (@idx) {
+      return $idx[0];
+   } else {
+      return -1;
+   }
+}
+
+sub rindex {
    my($self,$val) = @_;
    my @idx = $self->index($val);
-   return undef  if ($self->err());
 
-   return 1  if ($#idx > -1);
-   return 0;
+   if (wantarray) {
+      return CORE::reverse(@idx);
+   } elsif (@idx) {
+      return $idx[$#idx];
+   } else {
+      return -1;
+   }
 }
 
 sub is_empty {
    my($self,$undef) = @_;
    $$self{"err"} = "";
 
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
    my @list = @{ $$self{"set"} };
    return 1  if ($#list == -1);
 
@@ -218,26 +256,43 @@ sub is_empty {
    return 1;
 }
 
-sub as_hash {
+sub length {
    my($self) = @_;
    $$self{"err"} = "";
 
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
+   return $#{ $$self{"set"} } + 1;
+}
 
-   my %tmp;
-   foreach my $ele (@{ $$self{"set"} }) {
-      next  if (! defined $ele);
-      if (exists $tmp{$ele}) {
-         $tmp{$ele}++;
-      } else {
-         $tmp{$ele} = 1;
-      }
-   }
+sub list {
+   my($self,@list) = @_;
+   $$self{"err"}   = "";
 
-   return %tmp;
+   if (@list) {
+      $$self{"set"} = [@list];
+      return;
+   } else {
+      return @{ $$self{"set"} };
+   }
+}
+
+sub _eq {
+   my($self,$val1,$val2) = @_;
+
+   if (! defined $val1  &&  ! defined $val2) {
+      return 1;
+   } elsif (! defined $val1  ||  ! defined $val2) {
+      return 0;
+
+   } elsif (ref($val1)  &&  ref($val2)  &&  scalar($val1) eq scalar($val2)) {
+      return 1;
+   } elsif (ref($val1)  ||  ref($val2)) {
+      return 0;
+
+   } elsif ($val1 eq $val2) {
+      return 1;
+   } else {
+      return 0;
+   }
 }
 
 ###############################################################################
@@ -249,10 +304,6 @@ sub clear {
    $$self{"err"}   = "";
 
    if ($undef) {
-      if (! defined $$self{"set"}) {
-         $$self{"err"} = "List undefined";
-         return undef;
-      }
       foreach my $ele (@{ $$self{"set"} }) {
          $ele = undef;
       }
@@ -268,11 +319,6 @@ sub compact {
    my($self) = @_;
    $$self{"err"} = "";
 
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
    my @list = ();
    foreach my $ele (@{ $$self{"set"} }) {
       CORE::push(@list,$ele)  if (defined $ele);
@@ -281,202 +327,71 @@ sub compact {
    return;
 }
 
-sub unique {
-   my($self) = @_;
+sub delete {
+   my($self,$all,$undef,@val) = @_;
    $$self{"err"} = "";
 
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
+   foreach my $val (@val) {
+      my(@idx);
+      if ($all) {
+         @idx = $self->rindex($val);
+         next  if (! @idx);
+      } else {
+         my $idx = $self->index($val);
+         next  if ($idx == -1);
+         @idx = ($idx);
+      }
 
-   my @list = ();
-   my %list = ();
-   my $undef = 0;
-
-   foreach my $ele (@{ $$self{"set"} }) {
-      if (! defined($ele)) {
-         if (! $undef) {
-            CORE::push(@list,$ele);
-            $undef = 1;
+      if ($undef) {
+         foreach my $idx (@idx) {
+            $$self{"set"}[$idx] = undef;
+         }
+      } else {
+         foreach my $idx (@idx) {
+            CORE::splice(@{ $$self{"set"} },$idx,1);
          }
-      } elsif (! CORE::exists $list{$ele}) {
-         CORE::push(@list,$ele);
-         $list{$ele} = 1;
       }
    }
-   $$self{"set"} = [@list];
    return;
 }
 
-sub push {
-   my($self,@list) = @_;
-   $$self{"err"}   = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   CORE::push(@{ $$self{"set"} },@list);
-   return;
-}
-
-sub pop {
-   my($self) = @_;
-   $$self{"err"}   = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   my $val = CORE::pop @{ $$self{"set"} };
-   return $val;
-}
-
-sub unshift {
-   my($self,@list) = @_;
-   $$self{"err"}   = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   CORE::unshift(@{ $$self{"set"} },@list);
-   return;
-}
-
-sub shift {
-   my($self) = @_;
-   $$self{"err"}   = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   my $val = CORE::shift @{ $$self{"set"} };
-   return $val;
-}
-
-sub reverse {
-   my($self) = @_;
-   $$self{"err"}   = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   my @list = @{ $$self{"set"} };
-   $$self{"set"} = [ CORE::reverse(@list) ];
-   return;
-}
-
-sub rotate {
-   my($self,$n) = @_;
-   $n = 1  if (! defined $n);
+sub delete_at {
+   my($self,$undef,@idx) = @_;
    $$self{"err"} = "";
 
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   } elsif ($n !~ /^[+-]?\d+$/) {
-      $$self{"err"} = "Rotation number must be an integer";
-      return undef;
-   }
-
    my @list = @{ $$self{"set"} };
-   if ($n > 0) {
-      for (my $i=1; $i<=$n; $i++) {
-         CORE::push(@list,CORE::shift(@list));
+   foreach my $idx (@idx) {
+      if ($idx !~ /^[+-]?\d+$/) {
+         $$self{"err"} = "Index must be an integer";
+         return undef;
       }
-   } elsif ($n < 0) {
-      $n *= -1;
-      for (my $i=1; $i<=$n; $i++) {
-         CORE::unshift(@list,CORE::pop(@list));
+      if ($idx < -($#list + 1)  ||
+          $idx > $#list) {
+         $$self{"err"} = "Index out of bounds";
+         return undef;
+      }
+      if ($idx < 0) {
+         $idx = $#list + 1 + $idx;
       }
    }
+   @idx = sort { $b <=> $a } @idx;
 
-   $$self{"set"} = [@list];
+   if ($undef) {
+      foreach my $idx (@idx) {
+         $$self{"set"}[$idx] = undef;
+      }
+   } else {
+      foreach my $idx (@idx) {
+         CORE::splice(@{ $$self{"set"} },$idx,1);
+      }
+   }
    return;
 }
 
-sub sort {
-   my($self,$method,@args) = @_;
-
-   if (! defined $method) {
-      $method = "alphabetic";
-   }
-
-   my(@list) = _sort($self,$method,@args);
-   return undef  if ($self->err());
-
-   $$self{"set"} = [@list];
-   return;
-}
-
-sub _sort {
-   my($self,$method,@args) = @_;
-   $$self{"err"} = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   if (! sort_valid_method($method)) {
-      $$self{"err"} = "Invalid sort method";
-      return undef;
-   }
-
-   my @list = @{ $$self{"set"} };
-   sort_by_method($method,\@list,@args);
-   return @list;
-}
-
-sub randomize {
-   my($self) = @_;
-   $self->sort("random");
-}
-
-sub min {
-   my($self,$method,@args) = @_;
-
-   if (! defined $method) {
-      $method = "numerical";
-   }
-
-   my(@list) = _sort($self,$method,@args);
-   return undef  if ($self->err());
-
-   return $list[0];
-}
-
-sub max {
-   my($self,$method,@args) = @_;
-
-   if (! defined $method) {
-      $method = "numerical";
-   }
-
-   my(@list) = _sort($self,$method,@args);
-   return undef  if ($self->err());
-
-   return $list[$#list];
-}
-
 sub fill {
    my($self,$val,$start,$length) = @_;
    $$self{"err"} = "";
 
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
    my @list = @{ $$self{"set"} };
 
    $start = 0  if (! $start);
@@ -515,15 +430,161 @@ sub fill {
    return;
 }
 
+sub min {
+   my($self,$method,@args) = @_;
+
+   if (! defined $method) {
+      $method = "numerical";
+   }
+
+   my(@list) = _sort($self,$method,@args);
+   return undef  if ($self->err());
+
+   return $list[0];
+}
+
+sub max {
+   my($self,$method,@args) = @_;
+
+   if (! defined $method) {
+      $method = "numerical";
+   }
+
+   my(@list) = _sort($self,$method,@args);
+   return undef  if ($self->err());
+
+   return $list[$#list];
+}
+
+sub pop {
+   my($self) = @_;
+   $$self{"err"}   = "";
+
+   my $val = CORE::pop @{ $$self{"set"} };
+   return $val;
+}
+
+sub shift {
+   my($self) = @_;
+   $$self{"err"}   = "";
+
+   my $val = CORE::shift @{ $$self{"set"} };
+   return $val;
+}
+
+sub push {
+   my($self,@list) = @_;
+   $$self{"err"}   = "";
+
+   CORE::push(@{ $$self{"set"} },@list);
+   return;
+}
+
+sub unshift {
+   my($self,@list) = @_;
+   $$self{"err"}   = "";
+
+   CORE::unshift(@{ $$self{"set"} },@list);
+   return;
+}
+
+sub randomize {
+   my($self) = @_;
+   $self->sort("random");
+}
+
+sub reverse {
+   my($self) = @_;
+   $$self{"err"}   = "";
+
+   my @list = @{ $$self{"set"} };
+   $$self{"set"} = [ CORE::reverse(@list) ];
+   return;
+}
+
+sub rotate {
+   my($self,$n) = @_;
+   $n = 1  if (! defined $n);
+   $$self{"err"} = "";
+
+   if ($n !~ /^[+-]?\d+$/) {
+      $$self{"err"} = "Rotation number must be an integer";
+      return undef;
+   }
+
+   my @list = @{ $$self{"set"} };
+   if ($n > 0) {
+      for (my $i=1; $i<=$n; $i++) {
+         CORE::push(@list,CORE::shift(@list));
+      }
+   } elsif ($n < 0) {
+      $n *= -1;
+      for (my $i=1; $i<=$n; $i++) {
+         CORE::unshift(@list,CORE::pop(@list));
+      }
+   }
+
+   $$self{"set"} = [@list];
+   return;
+}
+
+sub set {
+   my($self,$index,$val) = @_;
+   $$self{"err"} = "";
+
+   if (! defined $index) {
+      $$self{"err"} = "Index required";
+      return undef;
+   }
+
+   my @list = @{ $$self{"set"} };
+
+   if ($index !~ /^[+-]?\d+$/) {
+      $$self{"err"} = "Index must be an integer";
+      return undef;
+   }
+   if ($index < -($#list + 1)  ||
+       $index > $#list) {
+      $$self{"err"} = "Index out of bounds";
+      return undef;
+   }
+
+   $$self{"set"}[$index] = $val;
+   return;
+}
+
+sub sort {
+   my($self,$method,@args) = @_;
+
+   if (! defined $method) {
+      $method = "alphabetic";
+   }
+
+   my(@list) = _sort($self,$method,@args);
+   return undef  if ($self->err());
+
+   $$self{"set"} = [@list];
+   return;
+}
+
+sub _sort {
+   my($self,$method,@args) = @_;
+   $$self{"err"} = "";
+
+   if (! sort_valid_method($method)) {
+      $$self{"err"} = "Invalid sort method";
+      return undef;
+   }
+
+   my @list = @{ $$self{"set"} };
+   sort_by_method($method,\@list,@args);
+   return @list;
+}
+
 sub splice {
    my($self,$start,$length,@vals) = @_;
    $$self{"err"} = "";
 
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
    my @list = @{ $$self{"set"} };
 
    $start = 0  if (! $start);
@@ -560,101 +621,26 @@ sub splice {
    return @ret;
 }
 
-sub set {
-   my($self,$index,$val) = @_;
+sub unique {
+   my($self) = @_;
    $$self{"err"} = "";
 
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-   if (! defined $index) {
-      $$self{"err"} = "Index required";
-      return undef;
-   }
+   my @list = ();
+   my %list = ();
+   my $undef = 0;
 
-   my @list = @{ $$self{"set"} };
-
-   if ($index !~ /^[+-]?\d+$/) {
-      $$self{"err"} = "Index must be an integer";
-      return undef;
-   }
-   if ($index < -($#list + 1)  ||
-       $index > $#list) {
-      $$self{"err"} = "Index out of bounds";
-      return undef;
-   }
-
-   $$self{"set"}[$index] = $val;
-   return;
-}
-
-sub delete {
-   my($self,$all,$undef,@val) = @_;
-   $$self{"err"} = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   foreach my $val (@val) {
-      my(@idx);
-      if ($all) {
-         @idx = $self->rindex($val);
-      } else {
-         my $idx = $self->index($val);
-         @idx = ($idx);
-      }
-
-      if ($undef) {
-         foreach my $idx (@idx) {
-            $$self{"set"}[$idx] = undef;
-         }
-      } else {
-         foreach my $idx (@idx) {
-            CORE::splice(@{ $$self{"set"} },$idx,1);
+   foreach my $ele (@{ $$self{"set"} }) {
+      if (! defined($ele)) {
+         if (! $undef) {
+            CORE::push(@list,$ele);
+            $undef = 1;
          }
+      } elsif (! CORE::exists $list{$ele}) {
+         CORE::push(@list,$ele);
+         $list{$ele} = 1;
       }
    }
-   return;
-}
-
-sub delete_at {
-   my($self,$undef,@idx) = @_;
-   $$self{"err"} = "";
-
-   if (! defined $$self{"set"}) {
-      $$self{"err"} = "List undefined";
-      return undef;
-   }
-
-   my @list = @{ $$self{"set"} };
-   foreach my $idx (@idx) {
-      if ($idx !~ /^[+-]?\d+$/) {
-         $$self{"err"} = "Index must be an integer";
-         return undef;
-      }
-      if ($idx < -($#list + 1)  ||
-          $idx > $#list) {
-         $$self{"err"} = "Index out of bounds";
-         return undef;
-      }
-      if ($idx < 0) {
-         $idx = $#list + 1 + $idx;
-      }
-   }
-   @idx = sort { $b <=> $a } @idx;
-
-   if ($undef) {
-      foreach my $idx (@idx) {
-         $$self{"set"}[$idx] = undef;
-      }
-   } else {
-      foreach my $idx (@idx) {
-         CORE::splice(@{ $$self{"set"} },$idx,1);
-      }
-   }
+   $$self{"set"} = [@list];
    return;
 }
 
@@ -662,40 +648,10 @@ sub delete_at {
 # SET METHODS
 ###############################################################################
 
-sub union {
-   my($obj1,$obj2,$unique) = @_;
-
-   my $class = ref($obj1);
-   my $ret   = new $class;
-
-   if (ref($obj2) ne $class) {
-      $$ret{"err"} = "Obj2 not of the right class";
-      return $ret;
-   }
-
-   if (! defined $$obj1{"set"}) {
-      $$ret{"err"} = "List undefined in obj1";
-      return $ret;
-   }
-   if (! defined $$obj2{"set"}) {
-      $$ret{"err"} = "List undefined in obj2";
-      return $ret;
-   }
-
-   my @list1 = @{ $$obj1{"set"} };
-   my @list2 = @{ $$obj2{"set"} };
-
-   $ret->list(@list1,@list2);
-   if ($unique) {
-      $ret->unique();
-   }
-
-   return $ret;
-}
-
 sub difference {
    my($obj1,$obj2,$unique) = @_;
 
+   my @list  = @{ $$obj1{"set"} };
    my $class = ref($obj1);
    my $ret   = new $class;
 
@@ -704,36 +660,15 @@ sub difference {
       return $ret;
    }
 
-   if (! defined $$obj1{"set"}) {
-      $$ret{"err"} = "List undefined in obj1";
-      return $ret;
-   }
-   if (! defined $$obj2{"set"}) {
-      $$ret{"err"} = "List undefined in obj2";
-      return $ret;
-   }
-
-   my @list1 = @{ $$obj1{"set"} };
-   my %list2 = $obj2->as_hash();
-   my $undef = $obj2->count();
-
-   my @list;
-   foreach my $ele (@list1) {
-      if (! defined $ele  &&  $undef > 0) {
-         next  if ($unique);
-         $undef--;
-         next;
-
-      } elsif (CORE::exists $list2{$ele}  &&  $list2{$ele}) {
-         next  if ($unique);
-         $list2{$ele}--;
-         next;
-      }
-
-      CORE::push(@list,$ele);
-   }
+   # $ret starts as identical to $obj1
+   # remove every element in $obj2 from $ret
 
    $ret->list(@list);
+   my $all = ($unique ? 1 : 0);
+   foreach my $ele (@{ $$obj2{"set"} }) {
+      $ret->delete($all,0,$ele);
+   }
+
    return $ret;
 }
 
@@ -748,32 +683,87 @@ sub intersection {
       return $ret;
    }
 
-   if (! defined $$obj1{"set"}) {
-      $$ret{"err"} = "List undefined in obj1";
-      return $ret;
-   }
-   if (! defined $$obj2{"set"}) {
-      $$ret{"err"} = "List undefined in obj2";
-      return $ret;
-   }
+   # $tmp is identical to $obj2
+   # foreach element in $obj1
+   #    if it's in $tmp
+   #       add it to $ret
+   #       remove it from $tmp
 
-   my @list1  = @{ $$obj1{"set"} };
-   my %list2  = $obj2->as_hash();
-   my $undef  = $obj2->count();
+   my $tmp   = new $class;
+   $tmp->list(@{ $$obj2{"set"} });
+   my $all   = ($unique ? 1 : 0);
 
-   my @list;
-   foreach my $ele (@list1) {
-      if ( (! defined $ele  &&  $undef)  ||
-           (defined $ele    &&  CORE::exists $list2{$ele}) ) {
-         CORE::push(@list,$ele);
+   my @list  = @{ $$obj1{"set"} };
+   foreach my $ele (@list) {
+      if ($tmp->exists($ele)) {
+         $ret->push($ele);
+         $tmp->delete($all,0,$ele);
       }
    }
 
-   $ret->list(@list);
-   $ret->unique()  if ($unique);
    return $ret;
 }
 
+sub is_equal {
+   my($obj1,$obj2,$unique) = @_;
+
+   my $class = ref($obj1);
+
+   if (ref($obj2) ne $class) {
+      return undef;
+   }
+
+   my @list1  = @{ $$obj1{"set"} };
+   my @list2  = @{ $$obj2{"set"} };
+
+   if ($unique) {
+      foreach my $ele (@list1) {
+         return 0  if (! $obj2->exists($ele));
+      }
+      foreach my $ele (@list2) {
+         return 0  if (! $obj1->exists($ele));
+      }
+      return 1;
+   }
+
+   foreach my $ele (@list1,@list2) {
+      return 0  if ($obj1->count($ele) != $obj2->count($ele));
+   }
+   return 1;
+}
+
+sub not_equal {
+   return 1 - is_equal(@_);
+}
+
+sub is_subset {
+   my($obj1,$obj2,$unique) = @_;
+
+   my $class = ref($obj1);
+
+   if (ref($obj2) ne $class) {
+      return undef;
+   }
+
+   my @list  = @{ $$obj2{"set"} };
+
+   if ($unique) {
+      foreach my $ele (@list) {
+         return 0  if (! $obj1->exists($ele));
+      }
+      return 1;
+   }
+
+   foreach my $ele (@list) {
+      return 0  if ($obj2->count($ele) > $obj1->count($ele));
+   }
+   return 1;
+}
+
+sub not_subset {
+   return 1 - is_subset(@_);
+}
+
 sub symmetric_difference {
    my($obj1,$obj2,$unique) = @_;
 
@@ -785,100 +775,52 @@ sub symmetric_difference {
       return $ret;
    }
 
-   if (! defined $$obj1{"set"}) {
-      $$ret{"err"} = "List undefined in obj1";
-      return $ret;
-   }
-   if (! defined $$obj2{"set"}) {
-      $$ret{"err"} = "List undefined in obj2";
-      return $ret;
-   }
+   my $tmp1  = new $class;
+   my @list1 = @{ $$obj1{"set"} };
+   $tmp1->list(@list1);
 
-   my @list1  = @{ $$obj1{"set"} };
-   my @list2  = @{ $$obj2{"set"} };
-   my %list1  = $obj1->as_hash();
-   my %list2  = $obj2->as_hash();
-   my $undef1 = $obj1->count();
-   my $undef2 = $obj2->count();
+   my $tmp2  = new $class;
+   my @list2 = @{ $$obj2{"set"} };
+   $tmp2->list(@list2);
 
-   my @list;
-   if ($unique) {
-      foreach my $ele (@list1) {
-         next  if (! defined $ele  &&  $undef2);
-         next  if (defined $ele    &&  CORE::exists $list2{$ele});
-         CORE::push(@list,$ele);
-      }
-      foreach my $ele (@list2) {
-         next  if (! defined $ele  &&  $undef1);
-         next  if (defined $ele    &&  CORE::exists $list1{$ele});
-         CORE::push(@list,$ele);
-      }
+   my $all   = ($unique ? 1 : 0);
 
-   } else {
-      foreach my $ele (@list1) {
-         $undef2--, next       if (! defined $ele  &&
-                                   $undef2);
-         $list2{$ele}--, next  if (defined $ele  &&
-                                   CORE::exists $list2{$ele}  &&
-                                   $list2{$ele});
-         CORE::push(@list,$ele);
-      }
-      foreach my $ele (@list2) {
-         $undef1--, next       if (! defined $ele  &&
-                                   $undef1);
-         $list1{$ele}--, next  if (defined $ele  &&
-                                   CORE::exists $list1{$ele}  &&
-                                   $list1{$ele});
-         CORE::push(@list,$ele);
+   foreach my $ele (@list1,@list2) {
+      if ($tmp1->exists($ele)  &&  $tmp2->exists($ele)) {
+         $tmp1->delete($all,0,$ele);
+         $tmp2->delete($all,0,$ele);
+      } elsif ($tmp1->exists($ele)) {
+         $ret->push($ele);
+         $tmp1->delete(0,0,$ele);
+      } elsif ($tmp2->exists($ele)) {
+         $ret->push($ele);
+         $tmp2->delete(0,0,$ele);
       }
    }
 
-   $ret->list(@list);
    return $ret;
 }
 
-sub is_equal {
+sub union {
    my($obj1,$obj2,$unique) = @_;
 
    my $class = ref($obj1);
+   my $ret   = new $class;
 
-   if (ref($obj2) ne $class  ||
-       ! defined $$obj1{"set"}  ||
-       ! defined $$obj2{"set"}) {
-      return undef;
+   if (ref($obj2) ne $class) {
+      $$ret{"err"} = "Obj2 not of the right class";
+      return $ret;
    }
 
-   my %list1  = $obj1->as_hash();
-   my %list2  = $obj2->as_hash();
-   my $undef1 = $obj1->count();
-   my $undef2 = $obj2->count();
+   my @list1 = @{ $$obj1{"set"} };
+   my @list2 = @{ $$obj2{"set"} };
 
+   $ret->list(@list1,@list2);
    if ($unique) {
-      return 0  if (($undef1  &&  ! $undef2) ||
-                    (! $undef1  &&  $undef2));
-      foreach my $ele (keys %list1) {
-         return 0  if (! exists $list2{$ele});
-      }
-      foreach my $ele (keys %list2) {
-         return 0  if (! exists $list1{$ele});
-      }
-
-   } else {
-      return 0  if ($undef1 != $undef2);
-      foreach my $ele (keys %list1) {
-         return 0  if (! exists $list2{$ele}  ||
-                       $list1{$ele} != $list2{$ele});
-      }
-      foreach my $ele (keys %list2) {
-         return 0  if (! exists $list1{$ele});
-      }
+      $ret->unique();
    }
 
-   return 1;
-}
-
-sub not_equal {
-   return 1 - is_equal(@_);
+   return $ret;
 }
 
 1;
